//#ifndef __CINT__
#include "RooGlobalFunc.h"
//#endif
#include "RooRealVar.h"
#include "RooGenericPdf.h"
#include "RooArgList.h"
#include "RooFormulaVar.h"
#include "RooDataSet.h"
#include "RooGaussian.h"
#include "RooChebychev.h"
#include "RooExponential.h"
#include "RooBifurGauss.h"
#include "RooAddModel.h"
#include "RooProdPdf.h"
#include "TCanvas.h"
#include "RooPlot.h"
#include "RooHist.h"
#include "RooCBShape.h"
#include "RooPolynomial.h"
#include "RooBinning.h"
#include "TH1.h"
#include "TH2.h"
#include "RooAddPdf.h"
#include "RooProdPdf.h"
#include "RooFitResult.h"
#include "RooGenericPdf.h"
#include "RooLandau.h"
#include "TChain.h"
#include "TCanvas.h"
#include "TAxis.h"
#include "TStyle.h"
#include "TH1F.h"
#include "TAxis.h"
#include "TLine.h"
#include "RooPlot.h"
#include "RooCategory.h"
#include "RooSuperCategory.h"
#include "RooSimultaneous.h"
#include "RooNLLVar.h"
#include "TFile.h"
#include "RooFit.h"
#include "RooGaussModel.h"
#include "RooHistPdf.h"
#include "RooPolyVar.h"

#include "myRooGaussian.cpp"

using namespace RooFit ;
using namespace std;



void unbinned_Kpipi0_RS_2D_fit_updated()  
{
	
  // get the data in a TTree format
  TFile *f = new TFile("DtoKpipi0_RS_mc14ri_a_2Dfit.root");
  TTree *tree = (TTree*)f->Get("DstD0PiKPiPi0RS_withcut");
  
  // check we have the tree
  cout << "Entries = " << tree->GetEntries() << endl;

  //create both fit observables and get the data in RooFit land
  RooRealVar m("pi0_m_prefit","#it{m}(#it{#pi}^{0})", 0.1, 0.18, "GeV/#it{c}^{2}");
  RooRealVar errm("pi0_errm","#it{m}(#it{#pi}^{0}) error", 0, 0.015, "GeV/#it{c}^{2}");
  RooDataSet* dataxy = new RooDataSet("data","data",tree, RooArgSet(m, errm));

  
  //*  S i n g l e    G a u s s i o n     R e s o l u t i o n    f u n c t i o n
  //  ------------------------------------------------------------------------
  
  RooRealVar errm1("errm1","errm1", 0, 0.015, "GeV/#it{c}^{2}");
  RooRealVar errm2("errm2","errm2", 0, 0.015, "GeV/#it{c}^{2}");  
  RooRealVar errm3("errm3","errm3", 0, 0.015, "GeV/#it{c}^{2}");
  
  errm1.setRange("SB1",0,0.0041);
  errm2.setRange("SB2",0.0041,0.0055);
  errm3.setRange("SB3",0.0055,0.015);


  // mean: Create function f(y) = a0 + a1*y (errm range: 0,0.0041)
  RooRealVar p0("p0", "p0", 1.34010e-01, 0, 10); // 1.34010e-01  
  RooRealVar p1("p1", "p1", -2.72535e-01, 0, 10); // -2.72535e-01
  RooPolyVar mu("mu", "mean", errm1, RooArgSet(p0, p1));

  // // mean: Create function f(y) = a0 + a1*y + a2*y^2 (errm range: 0.0041,0.0055)
  RooRealVar p0("p0", "p0", 9.22697e-02, 0, 10); // 9.22697e-02  
  RooRealVar p1("p1", "p1", 1.68039e+01, 0, 10); // 1.68039e+01
  RooRealVar p2("p2", "p2", -1.68719e+03, 0, 10); // -1.68719e+03
  RooPolyVar mu("mu", "mean", errm2, RooArgSet(p0, p1, p2));

  // // mean: Create function f(y) = a0 + a1*y + a2*y^2 (errm range: 0.0041,0.0055)
  RooRealVar p0("p0", "p0", 1.31621e-01, 0, 10); // 1.31621e-01  
  RooRealVar p1("p1", "p1", 3.81109e-01, 0, 10); // 3.81109e-01
  RooPolyVar mu("mu", "mean", errm3, RooArgSet(p0, p1));

 
  //Sigma: Create function f(y) = 1 + a/x + b/x^2 
  RooRealVar a("a", "a", -1.54699e-03); // -1.54699e-03  
  RooRealVar b("b", "b", 1.15198e-05);  // 1.15198e-05
  RooFormulaVar sigma("sigma", "@0+@1+@2/@0", RooArgList(errm,a,b));

  myRooGaussian gauss("gauss", "1st gaussian PDF", m, mu, sigma);
  //RooGaussian gauss("gauss", "1st gaussian PDF", m, mu, sigma);


  // C o n s t r u c t  p d f   f o r   e r r o r   m a s s
  // -----------------------------------------------------------------
  
  // Construct a histogram pdf to describe the shape of the ErrM distribution
  RooDataSet datamerr("datamerr","datamerr", tree, RooArgSet(errm));

  RooDataHist *expHistDterr = datamerr.binnedClone();
  RooHistPdf pdfErr("pdfErr", "pdfErr", errm, *expHistDterr);
 

  // C o n s t r u c t   c o n d i t i o n a l   p r o d u c t   d e c a y 
   // -----------------------------------------------------------------------------
  
  // Construct production of conditional decay_dm(dt|dterr) with empirical pdfErr(dterr)
  RooProdPdf pdf("pdf", "pdf(m|sigma_m)*pdf(sigma_m)", pdfErr, RooFit::Conditional(gauss, m));

  
  
  //perform the fit
  pdf.fitTo(*dataxy, RooFit::Save(true), RooFit::Strategy(2));
  
 /*-----------------------------------PLOT THE RESULTS-------------------------------------*/

  //-----------------------Canvas and pad
  TCanvas *canvas = new TCanvas("canvas","canvas", 700, 800);  //  700, 800
 
  Double_t xlow, ylow, xup, yup;
  canvas->GetPad(0)->GetPadPar(xlow,ylow,xup,yup);
  canvas->Divide(1,2);

  TVirtualPad *upPad = canvas->GetPad(1);
  upPad->SetPad(xlow,ylow+0.25*(yup-ylow),xup,yup);
  
  TVirtualPad *dwPad = canvas->GetPad(2);
  dwPad->SetPad(xlow,ylow,xup,ylow+0.25*(yup-ylow));

  //------------------------Plot pi0_M_prefit projections
  RooPlot *xframe = m.frame();
  dataxy->plotOn(xframe);
  // const double nData = dataxy->sumEntries("", "SB2");
  // cout<<" nentries in data are = "<<nData<<endl;
  pdf.plotOn(xframe, ProjWData(*dataxy));
  pdf.paramOn(xframe);
  canvas->cd(1);
  pdf.plotOn(xframe,Components(gauss),LineStyle(kDashed),LineColor(kRed), ProjWData(*dataxy));
  xframe->Draw();

  //----------------------- pull distribution of pi0_M_prefit
  RooHist* hpull = xframe->pullHist();
  hpull->SetFillStyle(1001);
  hpull->SetFillColor(1);
  for(int i=0;i<hpull->GetN();++i) hpull->SetPointError(i,0.0,0.0,0.0,0.0);
  RooPlot* pullplot = m.frame(Title(" "));
  pullplot->addPlotable(hpull,"B");
  pullplot->SetYTitle("Pull mass");
  pullplot->SetMinimum(-4.);
  pullplot->SetMaximum(4.);
  pullplot->GetXaxis()->SetLabelSize(0.1);
  pullplot->GetYaxis()->SetLabelSize(0.07);
  canvas->cd(2);
  pullplot->Draw();

  //------------------------- adding reference line in pull distribution
  double xmin1 = 0.1; 
  double xmax1 = 0.18;
  TLine *line = new TLine(xmin1,0.0,xmax1,0.0);
  TLine *line1 = new TLine(xmin1,3.0,xmax1,3.0);
  TLine *line2 = new TLine(xmin1,-3.0,xmax1,-3.0);
 
  line->SetLineColor(kRed); 
  line->SetLineWidth(3); 
  line1->SetLineColor(kRed);
  line2->SetLineColor(kRed);
  line1->SetLineStyle(2);
  line2->SetLineStyle(2);
  line->Draw("SAME"); 
  line1->Draw("SAME");
  line2->Draw("SAME");
  
  

  //------------------------- Plot pi0_ErrM projections
  RooPlot *yframe = errm.frame();
  dataxy->plotOn(yframe);
  pdf.plotOn(yframe);

  canvas->Update();
  

  // TCanvas *c = new TCanvas("c", "c", 700, 800);
  // //canvas->cd(2);
  // gPad->SetLeftMargin(0.15);
  // yframe->GetYaxis()->SetTitleOffset(1.6);
  // yframe->Draw();
 

 
}

  